'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2010
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'


Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Security
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Common.Utilities
Imports DotNetNuke.Modules.Workflow
Imports System.Linq

Namespace DotNetNuke.Modules.Html

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The Settings ModuleSettingsBase is used to manage the 
    ''' settings for the HTML Module
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[leupold]	    08/12/2007	created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Partial Public Class Settings
        Inherits DotNetNuke.Entities.Modules.ModuleSettingsBase

        Protected Sub cboWorkflow_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cboWorkflow.SelectedIndexChanged
            If Not cboWorkflow.SelectedValue Is Nothing Then
                lblDescription.Text = Workflow.WorkflowSettingsController.DisplayWorkflowDescription(Integer.Parse(cboWorkflow.SelectedValue))
            End If
        End Sub


        Private Sub CheckIfManageWorkflowNeedsInstallation()
            rowWorkflowEnable.Visible = False

            If PortalSecurity.IsInRole(PortalSettings.AdministratorRoleName) Then
                If ModuleControlController.GetModuleControlByControlKey("WorkFlow", ModuleConfiguration.ModuleDefID) Is Nothing Then
                    'Dim package As DotNetNuke.Services.Installer.Packages.PackageInfo = DotNetNuke.Services.Installer.Packages.PackageController.GetPackageByName("OliverHine.CustomPermissionProvider")
                    'If package IsNot Nothing Then
                    rowWorkflowEnable.Visible = True
                    'Else
                    'If Me.UserInfo.IsSuperUser = True Then
                    'rowWorkflowEnable.Visible = True
                    'cmdWorkflowEnable.Text = "For Additional Workflows, Install my Enhanced Permission Provider"
                    'End If
                    'End If
                End If
            End If
        End Sub

        Private Sub EnableWorkflowManagementModule()
            If ModuleControlController.GetModuleControlByControlKey("WorkFlow", ModuleConfiguration.ModuleDefID) Is Nothing Then
                'Dim package As DotNetNuke.Services.Installer.Packages.PackageInfo = DotNetNuke.Services.Installer.Packages.PackageController.GetPackageByName("OliverHine.CustomPermissionProvider")
                'If package IsNot Nothing Then
                Dim ModuleDefID As Integer = GetModuleDefinition("DNN_HTML", "Text/HTML")
                DotNetNuke.Services.Upgrade.Upgrade.AddModuleControl(ModuleDefID, "Workflow", "Manage Workflow", "DesktopModules/HTML/ManageWorkflow.ascx", "", SecurityAccessLevel.Admin, 0)

                Response.Redirect(Request.RawUrl)
                'Else
                'Response.Redirect("http://oliverhine.com/DotNetNuke/Providers/CustomPermissionsProvider.aspx", True)
                'End If
            End If
        End Sub

#Region "Installation Helpers"
        Private Shared Function GetModuleDefinition(ByVal DesktopModuleName As String, ByVal ModuleDefinitionName As String) As Integer
            ' get desktop module 
            Dim objDesktopModule As DesktopModuleInfo = DesktopModuleController.GetDesktopModuleByModuleName(DesktopModuleName, Null.NullInteger)
            If objDesktopModule Is Nothing Then
                Return -1
            End If

            ' get module definition 
            Dim objModuleDefinition As Entities.Modules.Definitions.ModuleDefinitionInfo = Entities.Modules.Definitions.ModuleDefinitionController.GetModuleDefinitionByFriendlyName(ModuleDefinitionName, objDesktopModule.DesktopModuleID)
            If objModuleDefinition Is Nothing Then
                Return -1
            End If

            Return objModuleDefinition.ModuleDefID

        End Function
#End Region

#Region "Base Method Implementations"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LoadSettings loads the settings from the Database and displays them
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub LoadSettings()
            Try
                If Not Page.IsPostBack Then

                    'hide the workflow settings control if required
                    If WorkflowSettingsController.CanManageWorkflow() = False Then
                        dshWorkflow.Visible = False
                        tblWorkflow.Visible = False
                    End If

                    'CheckIfManageWorkflowNeedsInstallation()

                    Dim objHtml As New HtmlTextController
                    Dim objWorkflow As New WorkflowStateController

                    ' get replace token settings
                    If CType(ModuleSettings("HtmlText_ReplaceTokens"), String) <> "" Then
                        chkReplaceTokens.Checked = CType(ModuleSettings("HtmlText_ReplaceTokens"), Boolean)
                    End If

                    ' get jQuery UI settings
                    If CType(ModuleSettings("HtmlText_jQueryUI"), String) <> "" Then
                        chkJqueryUI.Checked = CType(ModuleSettings("HtmlText_jQueryUI"), Boolean)
                    End If
                    If chkJqueryUI.Checked Then
                        If CType(ModuleSettings("HtmlText_DisplayMode"), String) <> "" Then
                            Dim a As ListItem = ddlDisplayMethod.Items.FindByValue(ModuleSettings("HtmlText_DisplayMode").ToString)
                            If a IsNot Nothing Then
                                ddlDisplayMethod.ClearSelection()
                                a.Selected = True
                            End If
                        End If
                        If CType(ModuleSettings("HtmlText_DisplayStyle"), String) <> "" Then
                            If ModuleSettings("HtmlText_DisplayStyle").ToString.ToLower = "base" Then ModuleSettings("HtmlText_DisplayStyle") = "lightness"
                            Dim a As ListItem = ddlDisplayStyle.Items.FindByValue(ModuleSettings("HtmlText_DisplayStyle").ToString)
                            If a IsNot Nothing Then
                                ddlDisplayStyle.ClearSelection()
                                a.Selected = True
                            End If
                        End If
                    Else
                        rowJqueryUI.Visible = False
                    End If

                    'depending on the selected view mode, we need to hide or show the style list
                    Select Case ddlDisplayMethod.SelectedValue.ToLower()
                        Case "none"
                            ddlDisplayStyle.Visible = False
                        Case Else
                            ddlDisplayStyle.Visible = True
                    End Select

                    ' expose workflow management option if available and user is administrator
                    If WorkflowSettingsController.GetWorkflowModule(PortalId) IsNot Nothing Then
                        If PortalSecurity.IsInRole(PortalSettings.AdministratorRoleName) Then
                            rowWorkflow.Visible = True
                        End If
                    End If

                    ' get workflow/version settings
                    Dim arrWorkflows As New ArrayList
                    For Each objState As WorkflowStateInfo In objWorkflow.GetWorkflows(PortalId)
                        If Not objState.IsDeleted Then
                            arrWorkflows.Add(objState)
                        End If
                    Next
                    cboWorkflow.DataSource = arrWorkflows
                    cboWorkflow.DataBind()
                    Dim kvpWorkflows As KeyValuePair(Of String, Integer) = DotNetNuke.Modules.Workflow.WorkflowSettingsController.GetWorkflow(ModuleId, TabId, PortalId)
                    If Not cboWorkflow.Items.FindByValue(kvpWorkflows.Value.ToString()) Is Nothing Then
                        cboWorkflow.Items.FindByValue(kvpWorkflows.Value.ToString()).Selected = True
                    End If
                    If Not cboWorkflow.SelectedValue Is Nothing Then
                        lblDescription.Text = Workflow.WorkflowSettingsController.DisplayWorkflowDescription(Integer.Parse(cboWorkflow.SelectedValue))
                    End If

                    ' expose default option if user is administrator
                    If PortalSecurity.IsInRole(PortalSettings.AdministratorRoleName) Then
                        rowApplyTo.Visible = True
                        If Not rblApplyTo.Items.FindByValue(kvpWorkflows.Key) Is Nothing Then
                            rblApplyTo.Items.FindByValue(kvpWorkflows.Key).Selected = True
                        End If
                    End If

                End If
            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' UpdateSettings saves the modified settings to the Database
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub UpdateSettings()
            Try

                ' update replace token setting
                Dim objModules As New DotNetNuke.Entities.Modules.ModuleController
                objModules.UpdateModuleSetting(ModuleId, "HtmlText_ReplaceTokens", chkReplaceTokens.Checked.ToString)
                objModules.UpdateModuleSetting(ModuleId, "HtmlText_jQueryUI", chkJqueryUI.Checked.ToString)
                objModules.UpdateModuleSetting(ModuleId, "HtmlText_DisplayMode", ddlDisplayMethod.SelectedValue)
                objModules.UpdateModuleSetting(ModuleId, "HtmlText_DisplayStyle", ddlDisplayStyle.SelectedValue)

                ' disable module caching if token replace is enabled
                If chkReplaceTokens.Checked Then
                    Dim objModule As DotNetNuke.Entities.Modules.ModuleInfo = objModules.GetModule(ModuleId, TabId, False)
                    If objModule.CacheTime > 0 Then
                        objModule.CacheTime = 0
                        objModules.UpdateModule(objModule)
                    End If
                End If

                If chkJqueryUI.Checked Then
                    Dim objModule As DotNetNuke.Entities.Modules.ModuleInfo = objModules.GetModule(ModuleId, TabId, False)
                    If objModule.CacheTime > 0 Then
                        objModule.CacheTime = 0
                        objModules.UpdateModule(objModule)
                    End If
                End If

                ' update workflow/version settings
                Select Case rblApplyTo.SelectedValue
                    Case "Module"
                        Workflow.WorkflowSettingsController.UpdateWorkflow(ModuleId, rblApplyTo.SelectedValue, Integer.Parse(cboWorkflow.SelectedValue), chkReplace.Checked, "Text/HTML")
                    Case "Page"
                        Workflow.WorkflowSettingsController.UpdateWorkflow(TabId, rblApplyTo.SelectedValue, Integer.Parse(cboWorkflow.SelectedValue), chkReplace.Checked, "Text/HTML")
                    Case "Site"
                        Workflow.WorkflowSettingsController.UpdateWorkflow(PortalId, rblApplyTo.SelectedValue, Integer.Parse(cboWorkflow.SelectedValue), chkReplace.Checked, "Text/HTML")
                End Select

                'module caching can't be enabled if workflow is, so this will disable that setting if we have more then 1 workflow state
                WorkflowSettingsController.CheckAndDisableModuleCachingIfWorkflowIsEnabled(ModuleId, TabId, PortalId)

                'remove cached content
                Common.Utilities.DataCache.RemoveCache(String.Format("h{0}", Me.ModuleId, PortalSettings.UserMode))

                'Common.Utilities.DataCache.ClearModuleCache(TabId)
            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

#End Region

        Private Sub chkUSEjQueryUI_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkJqueryUI.CheckedChanged
            If chkJqueryUI.Checked = True Then
                rowJqueryUI.Visible = True
            Else
                rowJqueryUI.Visible = False
            End If
        End Sub

        Private Sub cmdWorkflow_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdWorkflow.Click
            If PortalSecurity.IsInRole(PortalSettings.AdministratorRoleName) Then
                Dim workflowModule As ModuleInfo = WorkflowSettingsController.GetWorkflowModule(PortalId)
                If workflowModule IsNot Nothing Then
                    Response.Redirect(NavigateURL(workflowModule.TabID))
                End If
            End If
        End Sub

        Private Sub cmdWorkflowEnable_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdWorkflowEnable.Click
            EnableWorkflowManagementModule()
        End Sub

        Private Sub ddlDisplayMethod_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ddlDisplayMethod.SelectedIndexChanged
            Select Case ddlDisplayMethod.SelectedValue.ToLower()
                Case "none"
                    ddlDisplayStyle.Visible = False
                Case Else
                    ddlDisplayStyle.Visible = True
            End Select
        End Sub
    End Class

End Namespace


